@@ -485,7 +485,8 @@ blog_lj_build_postevent_request (const gchar *username, const gchar *uri,
 		const gchar *event, const gchar *music, const gchar *mood,
 		gint moodid, const gchar *subject, const gchar *security, gint mask,
 		const gchar *picture, gint year, gint month, gint day, gint hour,
-		gint minute, gint nocomments, gint preformatted, const DrivelJournal *dj)
+		gint minute, gint nocomments, gint preformatted, 
+		const DrivelJournal *dj, gint backdate)
 {
 	DrivelRequest *dr;
 	const gchar *journal;
@@ -549,6 +550,10 @@ blog_lj_build_postevent_request (const gchar *username, const gchar *uri,
 	drivel_request_add_items (dr, 
 			curl_escape ("prop_opt_preformatted", 0), g_strdup (text), NULL);
 	
+	g_snprintf (text, 8, "%d", backdate);
+	drivel_request_add_items (dr,
+			curl_escape ("prop_opt_backdated", 0), g_strdup (text), NULL);
+	
 	if (journal)
 	{
 		drivel_request_add_items (dr, 
