@@ -170,13 +170,15 @@ history_response_cb (GtkWidget *widget, gint arg, gpointer data)
 	DrivelClient *dc = (DrivelClient *) data;
 
 	if (arg == GTK_RESPONSE_OK)
+	{
 		history_list_edit_cb (NULL, data);
-
-	drivel_pop_current_window (dc);
-	
-	gtk_widget_destroy (widget);
-	dc->edit_history_window = NULL;
-	
+		gtk_window_present (GTK_WINDOW (dc->journal_window));
+	}
+	else {
+		drivel_pop_current_window (dc);
+		gtk_widget_destroy (widget);
+		dc->edit_history_window = NULL;
+	}
 	return;
 }
 
@@ -1059,10 +1061,13 @@ image_response_cb (GtkWidget *widget, gint arg, gpointer data)
 		protocol = strstr (url, "://");
 		if (protocol)
 		{
+			gchar *tmp;
+			
 			prefix = g_strdup (url);
 			prefix[strlen (url) - strlen (protocol)] = '\0';
+			tmp = g_strdup (protocol + 3);
 			g_free (url);
-			url = g_strdup (protocol + 3);
+			url = tmp;
 		}
 		else
 			prefix = g_strdup ("http");
@@ -1335,7 +1340,12 @@ display_edit_history_dialog (DrivelClient *dc)
 		
 		return;
 	}
+	if (dc->edit_history_window)
+	{
+		gtk_window_present(GTK_WINDOW (dc->edit_history_window));
 
+		return;
+	}
 	xml = load_glade_xml ("history_dialog");
 	if (!xml)
 		return;
