@@ -13,7 +13,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- * Copyright 2002-2004 Todd Kulesza
+ * Copyright 2002-2005 Todd Kulesza
  *
  * Authors:
  * 		Todd Kulesza <todd@dropline.net>
@@ -32,10 +32,13 @@
 #include "msg_queue.h"
 #include "login.h"
 #include "journal.h"
-#include "md5.h"
+#include "xmlrpc.h"
 #include "network.h"
 
+#define USER_AGENT "GNOME-Drivel/" VERSION
+
 extern GMutex *net_mutex;
+extern gboolean verbose;
 static GAsyncQueue *net_queue;
 
 static void
@@ -70,6 +73,25 @@ write_cb (void *ptr, size_t size, size_t nmemb, void *data)
 	return realsize;
 }
 
+static size_t
+read_cb (gchar *buffer, size_t size, size_t nmemb, void *data)
+{
+	gchar *xml_copied, *remaining_data;
+	gint bytes_copied = 0, maxsize = size *nmemb;
+	DrivelRequestData *memory = data;
+	
+	if (memory->data)
+	{
+		xml_copied = strncpy (buffer, memory->data, maxsize);
+		bytes_copied = strlen (xml_copied);
+		remaining_data = g_strdup (memory->data + bytes_copied);
+		g_free (memory->data);
+		memory->data = remaining_data;
+	}
+
+	return bytes_copied;
+}
+
 static gchar*
 get_line (const gchar *data, gint *offset)
 {
@@ -183,7 +205,7 @@ static void
 setup_proxies (DrivelClient *dc, CURL *session, gchar **proxy_userpwd, gchar **proxy_url)
 {
 	gchar *user, *pass, *url, *userpwd;
-	gboolean use_proxy;
+	gboolean use_proxy, use_auth;
 	gint port;
 	
 	g_mutex_lock (net_mutex);
@@ -197,20 +219,48 @@ setup_proxies (DrivelClient *dc, CURL *session, gchar **proxy_userpwd, gchar **p
 	else
 		pass = NULL;
 	userpwd = NULL;
-	url = g_strdup (dc->proxy_url);
+	if (dc->proxy_url)
+	{
+		gchar *colon1, *colon2;
+		
+		/* Check for an IPv6 URL by searching for at least two colons (:) */
+		colon1 = strstr (dc->proxy_url, ":");
+		if (colon1)
+			colon2 = strstr (colon1 + 1, ":");
+		else
+			colon2 = NULL;
+		if (colon2)
+			url = g_strdup_printf ("[%s]", dc->proxy_url);
+		else
+			url = g_strdup (dc->proxy_url);
+	}
+	else
+		url = NULL;
 	port = dc->proxy_port;
+	use_auth = dc->proxy_auth;
 	g_mutex_unlock (net_mutex);
 
 	/* Proxy stuff */
 	if (use_proxy)
 	{
-		if (user && user [0] != '\0' && pass && pass [0] != '\0')
+		gchar *msg;
+		
+		/* Newer releases of GNOME use the "use_authentication" GConf key
+		 * to enable/disable authentication, older releases just cleared the
+		 * username */
+		if (use_auth && user && user [0] != '\0' && pass && pass [0] != '\0')
 		{
 			userpwd = g_strdup_printf ("%s:%s", user, pass);	
 			curl_easy_setopt (session, CURLOPT_PROXYUSERPWD, userpwd);
 		}
-		/* we listen to GConf for changes to the proxy URL and port, so these
+		/* We listen to GConf for changes to the proxy URL and port, so these
 		   variables should always be up to date */
+		msg = g_strdup_printf (
+				"Proxy enabled:\nURL: %s\nPort: %d\nAuthentication: %s\n",
+				url, port, userpwd);
+		debug (msg);
+		g_free (msg);
+		
 		curl_easy_setopt (session, CURLOPT_PROXY, url);
 		curl_easy_setopt (session, CURLOPT_PROXYPORT, port);
 	}
@@ -236,22 +286,10 @@ static gchar*
 get_response_lj (const gchar *pass, const gchar *challenge)
 {
 	gchar *response;
-	guchar hash[16 * 2 + 1];
-	gint i;
-	md5_state_t state;
-	md5_byte_t digest[16];
+	guchar *hash;
 	
 	response = g_strdup_printf ("%s%s", challenge, pass);
-	
-	/* hash the response */
-	md5_init (&state);
-	md5_append (&state, response, strlen (response));
-	md5_finish (&state, digest);
-
-	for (i = 0; i < 16; i++)
-		g_snprintf (hash + i * 2, 3, "%02x", digest [i]);
-	hash [16 * 2] = '\0';
-	
+	hash = md5_hash (response);
 	g_free (response);
 	
 	return g_strdup (hash);
@@ -277,6 +315,7 @@ get_challenge_lj (DrivelClient *dc, DrivelRequest *dr, const gchar *url,
 	cookie = g_strdup_printf ("ljfastserver=%d", fast_servers);
 	
 	session = curl_easy_init ();
+	curl_easy_setopt (session, CURLOPT_USERAGENT, USER_AGENT);
 	curl_easy_setopt (session, CURLOPT_WRITEFUNCTION, write_cb);
 	curl_easy_setopt (session, CURLOPT_WRITEDATA, memory);
 	curl_easy_setopt (session, CURLOPT_URL, url);
@@ -303,7 +342,7 @@ get_challenge_lj (DrivelClient *dc, DrivelRequest *dr, const gchar *url,
 	{
 		*challenge = NULL;
 		if (curl_code == CURLE_ABORTED_BY_CALLBACK)
-			retval = -2;
+			retval = 1;
 		else
 		{
 			g_warning ("Could not get a challenge token from the server.");
@@ -360,7 +399,13 @@ perform_post (DrivelClient *dc, DrivelRequest *dr, const gchar *url,
 	memory->data = NULL;
 	proxy_userpwd = proxy_url = NULL;
 	
+	debug ("post url:");
+	debug (url);
+	debug ("post field:");
+	debug (request_string);
+	
 	session = curl_easy_init ();
+	curl_easy_setopt (session, CURLOPT_USERAGENT, USER_AGENT);
 	curl_easy_setopt (session, CURLOPT_WRITEFUNCTION, write_cb);
 	curl_easy_setopt (session, CURLOPT_WRITEDATA, memory);
 	curl_easy_setopt (session, CURLOPT_URL, url);
@@ -379,17 +424,20 @@ perform_post (DrivelClient *dc, DrivelRequest *dr, const gchar *url,
 	
 	if (http_code == 200 && curl_code != CURLE_ABORTED_BY_CALLBACK)
 	{
+		debug ("post result:");
+		debug (memory->data);
+		
 		parse_to_hash_table (dc, dr, memory->data, memory->len);
 		retval = 0;
 	}
 	else
 	{
 		if (curl_code == CURLE_ABORTED_BY_CALLBACK)
-			retval = -2;
+			retval = 1;
 		else
 		{
 			retval = -1;
-			g_warning ("HTTP POST request failed.");
+			g_warning ("HTTP/POST request failed (HTTP code %d).", (gint)http_code);
 		}
 	}
 	
@@ -401,24 +449,29 @@ perform_post (DrivelClient *dc, DrivelRequest *dr, const gchar *url,
 	return retval;
 }
 
-/* perform the HTTP/POST operation */
+/* perform the HTTP/GET operation */
 static gint
 perform_get (DrivelClient *dc, DrivelRequest *dr, const gchar *url, 
-		const gchar *cookies)
+		const gchar *cookies, struct curl_slist *headers)
 {
 	CURL *session;
 	CURLcode curl_code;
 	DrivelRequestData *memory;
-	gchar *proxy_userpwd, *proxy_url;
+	gchar *proxy_userpwd, *proxy_url, *userpass;
+	const gchar *http_type;
 	glong http_code;
 	gint retval;
 	
 	memory = g_new (DrivelRequestData, 1);
 	memory->len = 0;
 	memory->data = NULL;
-	proxy_userpwd = proxy_url = NULL;
+	proxy_userpwd = proxy_url = userpass = NULL;
+	
+	debug ("get url:");
+	debug (url);
 	
 	session = curl_easy_init ();
+	curl_easy_setopt (session, CURLOPT_USERAGENT, USER_AGENT);
 	curl_easy_setopt (session, CURLOPT_WRITEFUNCTION, write_cb);
 	curl_easy_setopt (session, CURLOPT_WRITEDATA, memory);
 	curl_easy_setopt (session, CURLOPT_URL, url);
@@ -427,6 +480,101 @@ perform_get (DrivelClient *dc, DrivelRequest *dr, const gchar *url,
 	curl_easy_setopt (session, CURLOPT_PROGRESSDATA, dc);
 	curl_easy_setopt (session, CURLOPT_COOKIE, cookies);
 	curl_easy_setopt (session, CURLOPT_SHARE, dc->net->curl_share);
+	curl_easy_setopt (session, CURLOPT_HTTPHEADER, headers);
+	if (drivel_request_item_lookup (dr, "http_basic"))
+	{
+		const gchar *username, *password;
+		
+		username = drivel_request_item_lookup (dr, "username");
+		password = drivel_request_item_lookup (dr, "password");
+		userpass = g_strdup_printf ("%s:%s", username, password);
+		
+		curl_easy_setopt (session, CURLOPT_HTTPAUTH, CURLAUTH_BASIC);
+		curl_easy_setopt (session, CURLOPT_USERPWD, userpass);
+	}
+	http_type = drivel_request_item_lookup (dr, "http_type");
+	if (http_type && !strcmp (http_type, "delete"))
+		curl_easy_setopt (session, CURLOPT_CUSTOMREQUEST, "DELETE");
+	setup_proxies(dc, session, &proxy_userpwd, &proxy_url);
+	
+	curl_code = curl_easy_perform (session);
+	
+	http_code = 0;
+	curl_easy_getinfo (session, CURLINFO_HTTP_CODE, &http_code);
+	
+	if ((http_code == 200 || http_code == 204) && 
+		curl_code != CURLE_ABORTED_BY_CALLBACK)
+	{
+		debug ("get result:");
+		debug (memory->data);
+		
+		drivel_request_set_data (dr, memory);
+		retval = 0;
+	}
+	else
+	{
+		g_free (memory->data);
+		g_free (memory);
+		
+		if (curl_code == CURLE_ABORTED_BY_CALLBACK)
+			retval = 1;
+		else
+		{
+			retval = -1;
+			g_warning ("HTTP/GET request failed (HTTP code %d).", (gint)http_code);
+		}
+	}
+	
+	g_free (proxy_userpwd);
+	g_free (proxy_url);
+	g_free (userpass);
+	
+	return retval;
+}
+
+/* perform the HTTP/POST operation using XML-RPC */
+static gint
+perform_xmlrpc_post (DrivelClient *dc, DrivelRequest *dr, const gchar *url)
+{
+	CURL *session;
+	CURLcode curl_code;
+	struct curl_slist *headers;
+	DrivelRequestData *memory;
+	gchar *proxy_userpwd, *proxy_url;
+	const gchar *post_field;
+	glong http_code;
+	gint retval;
+	GHashTable *table;
+	
+	memory = g_new (DrivelRequestData, 1);
+	memory->len = 0;
+	memory->data = NULL;
+	headers = NULL;
+	headers = curl_slist_append (headers, "Content-Type: text/xml");
+	headers = curl_slist_append (headers, "Pragma:");
+	headers = curl_slist_append (headers, "Accept:");
+	proxy_userpwd = proxy_url = NULL;
+	post_field = drivel_request_item_lookup (dr, "xml");
+	
+	debug ("xmlrpc url:");
+	debug (url);
+	debug ("xmlrpc post field:");
+	debug (post_field);
+	
+	if (!post_field)
+		g_warning ("post_field is NULL");
+	
+	session = curl_easy_init ();
+	curl_easy_setopt (session, CURLOPT_USERAGENT, USER_AGENT);
+	curl_easy_setopt (session, CURLOPT_URL, url);
+	curl_easy_setopt (session, CURLOPT_POSTFIELDS, post_field);
+	curl_easy_setopt (session, CURLOPT_HTTPHEADER, headers);
+	curl_easy_setopt (session, CURLOPT_WRITEFUNCTION, write_cb);
+	curl_easy_setopt (session, CURLOPT_WRITEDATA, memory);
+	curl_easy_setopt (session, CURLOPT_NOPROGRESS, FALSE);
+	curl_easy_setopt (session, CURLOPT_PROGRESSFUNCTION, request_progress);
+	curl_easy_setopt (session, CURLOPT_PROGRESSDATA, dc);
+	curl_easy_setopt (session, CURLOPT_SHARE, dc->net->curl_share);
 	setup_proxies(dc, session, &proxy_userpwd, &proxy_url);
 	
 	curl_code = curl_easy_perform (session);
@@ -436,7 +584,11 @@ perform_get (DrivelClient *dc, DrivelRequest *dr, const gchar *url,
 	
 	if (http_code == 200 && curl_code != CURLE_ABORTED_BY_CALLBACK)
 	{
-		drivel_request_set_data (dr, memory);
+		debug ("xmlrpc result:");
+		debug (memory->data);
+		
+		table = xmlrpc_parse_packet (memory->data, memory->len);
+		drivel_request_set_values (dr, table);
 		retval = 0;
 	}
 	else
@@ -445,16 +597,117 @@ perform_get (DrivelClient *dc, DrivelRequest *dr, const gchar *url,
 		g_free (memory);
 		
 		if (curl_code == CURLE_ABORTED_BY_CALLBACK)
-			retval = -2;
+			retval = 1;
 		else
 		{
 			retval = -1;
-			g_warning ("HTTP GET request failed.");
+			g_warning ("HTTP/POST XML-RPC request failed (HTTP code %d).", (gint)http_code);
+		}
+	}
+	
+	curl_slist_free_all (headers);
+	g_free (proxy_userpwd);
+	g_free (proxy_url);
+	
+	return retval;
+}
+
+/* Perform an HTTP/POST operation with an Atom entry */
+static gint
+perform_atom_post (DrivelClient *dc, DrivelRequest *dr, 
+		struct curl_slist *headers)
+{
+	CURL *session;
+	CURLcode curl_code;
+	DrivelRequestData *wmemory, *rmemory;
+	gchar *proxy_userpwd, *proxy_url, *userpass;
+	const gchar *post_field;
+	glong http_code;
+	gint retval;
+	
+	wmemory = g_new (DrivelRequestData, 1);
+	wmemory->len = 0;
+	wmemory->data = NULL;
+	rmemory = g_new (DrivelRequestData, 1);
+	rmemory->len = 0;
+	rmemory->data = NULL;
+	
+	proxy_userpwd = proxy_url = userpass = NULL;
+	post_field = drivel_request_item_lookup (dr, "xml");
+	
+	debug ("atom post field:");
+	debug (post_field);
+	
+	if (!post_field)
+		g_warning ("post_field is NULL");
+	
+	session = curl_easy_init ();
+	curl_easy_setopt (session, CURLOPT_USERAGENT, USER_AGENT);
+	curl_easy_setopt (session, CURLOPT_URL, drivel_request_get_uri (dr));
+	curl_easy_setopt (session, CURLOPT_HTTPHEADER, headers);
+	curl_easy_setopt (session, CURLOPT_WRITEFUNCTION, write_cb);
+	curl_easy_setopt (session, CURLOPT_WRITEDATA, wmemory);
+	curl_easy_setopt (session, CURLOPT_NOPROGRESS, FALSE);
+	curl_easy_setopt (session, CURLOPT_PROGRESSFUNCTION, request_progress);
+	curl_easy_setopt (session, CURLOPT_PROGRESSDATA, dc);
+	curl_easy_setopt (session, CURLOPT_SHARE, dc->net->curl_share);
+	if (drivel_request_item_lookup (dr, "http_basic"))
+	{
+		const gchar *username, *password;
+		
+		username = drivel_request_item_lookup (dr, "username");
+		password = drivel_request_item_lookup (dr, "password");
+		userpass = g_strdup_printf ("%s:%s", username, password);
+		
+		curl_easy_setopt (session, CURLOPT_HTTPAUTH, CURLAUTH_BASIC);
+		curl_easy_setopt (session, CURLOPT_USERPWD, userpass);
+	}
+	if (drivel_request_item_lookup (dr, "http_put"))
+	{
+		rmemory->data = g_strdup (post_field);
+		curl_easy_setopt (session, CURLOPT_UPLOAD, TRUE);
+		curl_easy_setopt (session, CURLOPT_READFUNCTION, read_cb);
+		curl_easy_setopt (session, CURLOPT_READDATA, rmemory);
+	}
+	else
+		curl_easy_setopt (session, CURLOPT_POSTFIELDS, post_field);
+	setup_proxies(dc, session, &proxy_userpwd, &proxy_url);
+	
+	curl_code = curl_easy_perform (session);
+	
+	http_code = 0;
+	curl_easy_getinfo (session, CURLINFO_HTTP_CODE, &http_code);
+	
+	if (http_code == 200 && curl_code != CURLE_ABORTED_BY_CALLBACK)
+	{
+		debug ("atom result:");
+		debug (wmemory->data);
+		
+		/* FIXME: Parse the result */
+		retval = 0;
+	}
+	else
+	{
+		debug ("atom failed result:");
+		debug (wmemory->data);
+		
+		g_free (wmemory->data);
+		g_free (wmemory);
+		
+		if (curl_code == CURLE_ABORTED_BY_CALLBACK)
+			retval = 1;
+		else
+		{
+			retval = -1;
+			g_warning ("HTTP/POST Atom request failed (HTTP code %d).", (gint)http_code);
 		}
 	}
 	
 	g_free (proxy_userpwd);
 	g_free (proxy_url);
+	g_free (userpass);
+	g_free (rmemory->data);
+	g_free (rmemory);
 	
 	return retval;
 }
@@ -465,64 +718,102 @@ static void
 update_progress_msg (GAsyncQueue *queue, DrivelRequestType type)
 {
 	MsgInfo *info;
-	gchar *msg;
+	gchar *msg, *header;
 	
 	switch (type)
 	{
 		case REQUEST_TYPE_LOGIN:
 		{
-			msg = g_strdup ("Retrieving user information");
+			msg = g_strdup (_("Retrieving user information"));
 			break;
 		}
 		case REQUEST_TYPE_GETPICTURE:
 		{
-			msg = g_strdup ("Downloading user pictures");
+			msg = g_strdup (_("Downloading user pictures"));
 			break;
 		}
 		case REQUEST_TYPE_POSTEVENT:
 		{
-			msg = g_strdup ("Posting journal entry");
+			msg = g_strdup (_("Posting journal entry"));
 			break;
 		}
 		case REQUEST_TYPE_EDITEVENT:
 		{
-			msg = g_strdup ("Updating journal entry");
+			msg = g_strdup (_("Updating journal entry"));
 			break;
 		}
 		case REQUEST_TYPE_GETEVENTS:
 		{
-			msg = g_strdup ("Retrieving journal entries");
+			msg = g_strdup (_("Retrieving journal entries"));
 			break;
 		}
 		case REQUEST_TYPE_GETDAYCOUNTS:
 		{
-			msg = g_strdup ("Retrieving journal history");
+			msg = g_strdup (_("Retrieving journal history"));
 			break;
 		}
 		case REQUEST_TYPE_EDITFRIENDS:
 		{
-			msg = g_strdup ("Updating Friends list");
+			msg = g_strdup (_("Updating Friends list"));
 			break;
 		}
 		case REQUEST_TYPE_GETFRIENDS:
 		{
-			msg = g_strdup ("Retrieving Friends list");
+			msg = g_strdup (_("Retrieving Friends list"));
+			break;
+		}
+		case REQUEST_TYPE_GETCATEGORIES:
+		case REQUEST_TYPE_GETPOSTCATEGORIES:
+		{
+			msg = g_strdup (_("Retrieving categories"));
+			break;
+		}
+		case REQUEST_TYPE_SETPOSTCATEGORIES:
+		{
+			msg = g_strdup (_("Setting categories"));
+			break;
+		}
+		case REQUEST_TYPE_PUBLISH:
+		{
+			msg = g_strdup (_("Publishing journal entry"));
+			break;
+		}
+		case REQUEST_TYPE_DELETEEVENT:
+		{
+			msg = g_strdup (_("Deleting journal entry"));
+			break;
+		}
+		case REQUEST_TYPE_PING:
+		{
+			msg = g_strdup (_("Notifying Technorati"));
+			break;
+		}
+		case REQUEST_TYPE_GETFRIENDGROUPS:
+		{
+			msg = g_strdup (_("Retrieving security groups"));
+			break;
+		}
+		case REQUEST_TYPE_SETFRIENDGROUPS:
+		{
+			msg = g_strdup (_("Updating security groups"));
 			break;
 		}
 		default:
 		{
-			msg = g_strdup ("We're doing something, but I'm not sure what");
+			msg = g_strdup (_("We're doing something, but I'm not sure what"));
 			break;
 		}
 	}
 	
 	info = msg_info_new ();
 	info->type = MSG_TYPE_UPDATE_PROGRESS_LABEL;
+	header = g_strdup (_("Sending / Receiving"));
 	info->msg = g_strdup_printf ("<span weight=\"bold\" size=\"larger\">"
-			"Sending / Receiving</span>\n\n"
-			"%s...", msg);
+			"%s</span>\n\n"
+			"%s...", header, msg);
 	g_async_queue_push (queue, info);
 	
+	g_free (header);
 	g_free (msg);
 	
 	return;
@@ -541,11 +832,11 @@ net_thread (DrivelClient *dc)
 		MsgInfo *info;
 		gint retval = 0;
 		gchar *request_string = NULL, *cookies = NULL;
-		
+
 		dr = g_async_queue_pop (net_queue);
-		
+
 		type = drivel_request_get_type (dr);
-		/* for everything except CHECKFRIENDS, we reparent the dialog to
+		/* for everything except CHECKFRIENDS, we re-parent the dialog to
 		   ensure it is transient for the top window, and then update its
 		   message. */
 		if (type != REQUEST_TYPE_CHECKFRIENDS)
@@ -554,7 +845,6 @@ net_thread (DrivelClient *dc)
 			info->type = MSG_TYPE_REPARENT_DIALOG;
 			info->dc = (gpointer) dc;
 			g_async_queue_push (dc->net->msg_queue, info);
-			
 			update_progress_msg (dc->net->msg_queue, type);
 		}
 		
@@ -573,11 +863,11 @@ net_thread (DrivelClient *dc)
 						
 						g_mutex_lock (net_mutex);
 						fast_servers = dc->net->fast_servers;
-						url = g_strdup_printf ("http://%s/interface/flat", 
-								dc->net->server);
-						password = g_strdup (dc->password);
+						password = md5_hash (dc->user->password);
 						g_mutex_unlock (net_mutex);
 						
+						url = g_strdup_printf ("%s/interface/flat", 
+								drivel_request_get_uri (dr));
 						cookies = g_strdup_printf ("ljfastserver=%d", fast_servers);
 						retval = get_challenge_lj (dc, dr, url, fast_servers, &challenge);
 						response = get_response_lj (password, challenge);
@@ -588,7 +878,7 @@ net_thread (DrivelClient *dc)
 								NULL);
 						
 						g_free (password);
-						
+
 						break;
 					}
 					default:
@@ -603,10 +893,12 @@ net_thread (DrivelClient *dc)
 				{
 					retval = perform_post (dc, dr, url, request_string, cookies);
 				}
-				if (retval == -1)
+				if (retval < 0)
+				{
 					send_error (dc, _("Communication Error"),
 							_("There was a problem sending information to the "
 							"server.  Please try again later."));
+				}
 				
 				g_free (url);
 				
@@ -615,7 +907,9 @@ net_thread (DrivelClient *dc)
 			case REQUEST_PROTOCOL_GET:
 			{
 				const gchar *url;
-				
+				gchar *wsse = NULL;
+				struct curl_slist *headers = NULL;
+					
 				switch (drivel_request_get_api (dr))
 				{
 					case BLOG_API_LJ:
@@ -631,6 +925,10 @@ net_thread (DrivelClient *dc)
 						
 						break;
 					}
+					case BLOG_API_ATOM:
+					case BLOG_API_MT:
+						/* nothing special to do for these */
+						break;
 					default:
 					{
 						g_warning ("net_thread: Unknown journal API.");
@@ -638,25 +936,77 @@ net_thread (DrivelClient *dc)
 					}
 				}
 				
-				url = drivel_request_item_lookup (dr, "url");
-				retval = perform_get (dc, dr, url, cookies);
-				if (retval == -1)
+				url = drivel_request_get_uri (dr);
+				retval = perform_get (dc, dr, url, cookies, headers);
+				if (retval < 0)
+				{
 					send_error (dc, _("Communication Error"),
 							_("There was a problem receiving information from "
 							"the server.  Please try again later."));
+				}
+				
+				g_free (wsse);
+				curl_slist_free_all (headers);
 				
 				break;
 			}
 			case REQUEST_PROTOCOL_XMLRPC:
 			{
-				g_warning ("XMLRPC hasn't been implemented yet.");
-				retval = -1;
+				const gchar *url = NULL;
+				
+				switch (drivel_request_get_api (dr))
+				{
+					case BLOG_API_ADVOGATO:
+					{
+						url = g_strdup_printf ("%s/XMLRPC", 
+								drivel_request_get_uri (dr));
+						break;
+					}
+					case BLOG_API_BLOGGER:
+					case BLOG_API_MT:
+					case BLOG_API_GENERIC:
+					{
+						url = drivel_request_get_uri (dr);
+						break;
+					}
+					default:
+					{
+						g_warning ("net_thread: Unknown journal API.");
+						break;
+					}
+				}
+				
+				retval = perform_xmlrpc_post (dc, dr, url);
+				if (retval < 0)
+				{
+					send_error (dc, _("Communication Error"),
+							_("There was a problem sending information to the "
+							"server.  Please try again later."));
+				}
+				
+				break;
+			}
+			case REQUEST_PROTOCOL_ATOM:
+			{
+				struct curl_slist *headers = NULL;
+				
+				headers = curl_slist_append (headers, "Content-Type: text/xml");
+						
+				retval = perform_atom_post (dc, dr, headers);
+				if (retval < 0)
+				{
+					send_error (dc, _("Communication Error"),
+							_("There was a problem sending information to the "
+							"server.  Please try again later."));
+				}
+				
+				curl_slist_free_all (headers);
+				
 				break;
 			}
 			case REQUEST_PROTOCOL_NONE:
 			{
-				g_warning ("net_thread: No protocol was selected.");
-				retval = -1;
+				/* we don't do anything, just pass the DrivelRequest on */
 				break;
 			}
 		}
@@ -692,6 +1042,9 @@ net_enqueue_request (DrivelClient *dc, DrivelRequest *dr)
 	MsgInfo *info;
 	DrivelRequestType type;
 	
+	if (!dr)
+		return;
+	
 	/* first build the progress dialog */
 	type = drivel_request_get_type (dr);
 	if (type != REQUEST_TYPE_CHECKFRIENDS)
@@ -701,7 +1054,7 @@ net_enqueue_request (DrivelClient *dc, DrivelRequest *dr)
 		info->msg = g_strdup ("<span weight=\"bold\" size=\"larger\">"
 				"Sending / Receiving</span>\n\n"
 				"Begining network transaction...");
-		info->widget = drivel_get_current_window (dc->window_list);
+		info->widget = GTK_WINDOW (dc->current_window);
 		info->dc = (gpointer)dc;
 		g_async_queue_push (dc->net->msg_queue, info);
 	}
@@ -732,3 +1085,40 @@ net_start_thread (DrivelClient *dc)
 	
 	return retval;
 }
+
+/* send a ping to technorati */
+void
+net_ping_technorati (DrivelClient *dc)
+{
+	DrivelRequest *dr;
+	gchar *packet;
+	const gchar *name;
+	
+	debug ("net_ping_technorati ()");
+	
+	if (!gconf_client_get_bool (dc->client, dc->gconf->technorati, NULL))
+		return;
+	
+	/* Use the LiveJournal description as the Technorati title for LJ users */
+	if ((dc->user->api == BLOG_API_LJ) && 
+		(dc->active_journal->type == JOURNAL_TYPE_USER))
+		name = dc->active_journal->description;
+	else
+		name = dc->active_journal->name;
+	
+	packet = xmlrpc_build_packet ("weblogUpdates.ping", 
+			XMLRPC_TYPE_STRING, name,
+			XMLRPC_TYPE_STRING, dc->active_journal->uri_view,
+			-1);
+	
+	dr = drivel_request_new_with_items (REQUEST_TYPE_PING,
+			REQUEST_PROTOCOL_XMLRPC, 
+			BLOG_API_GENERIC, 
+			"http://rpc.technorati.com/rpc/ping",
+			g_strdup ("xml"), packet,
+			NULL);
+	
+	net_enqueue_request (dc, dr);
+	
+	return;
+}
