@@ -24,27 +24,19 @@
 
 #include "drivel.h"
 
-typedef gint TrayStatus;
-
-enum
-{
-	TRAY_NORMAL,
-	TRAY_UPDATED_FRIENDS
-};
+/* Turn on the LiveJournal Friends Page check */
 
 void
-tray_hide_entry (DrivelClient *dc);
+tray_turn_on (DrivelClient *dc);
+
+/* Turn off the LiveJournal Friends Page and remove the tray icon */
 
 void
-tray_show_entry (DrivelClient *dc);
+tray_turn_off (DrivelClient *dc);
+
+/* Display the tray icon */
 
 void
-tray_icon_update (TrayStatus status);
-
-void
-tray_create (DrivelClient *dc);
-
-void
-tray_destroy (DrivelClient *dc);
+tray_display (gboolean visible);
 
 #endif
